﻿Imports System.Net
Imports System.Net.Sockets
Imports System.Text

Public Class Main
   Private WithEvents Listener As UdpListener
   Dim UdpClientObj As UdpClient = Nothing

   Private Sub CmdClear_Click(sender As Object, e As EventArgs) Handles cmdClear.Click
      lbDatagrams.Items.Clear()
   End Sub

   Private Sub CmdReceive_Click(sender As Object, e As EventArgs) Handles cmdReceive.Click
      If Listener Is Nothing Then
         cmdReceive.Text = "Empfang beenden"
         Listener = New UdpListener
         Listener.Start(UdpClientObj, CbClearBuffer.Checked)
         FillClientData()
      Else
         Listener.Stopp()
         cmdReceive.Enabled = False
      End If
   End Sub

   Private Sub CmdXmit_Click(sender As Object, e As EventArgs) Handles cmdXmit.Click
      Dim data As Byte() = Encoding.ASCII.GetBytes(tbData.Text)
      If UdpClientObj.Client.Connected Then
         UdpClientObj.Send(data, data.Length)
      Else
         Dim Port As Integer
         If Not Int32.TryParse(tbXmitPort.Text, Port) Then
            ' lblError.Text = "Angabe zum lokalen Port ist ungültig."
            MsgBox("Port falsch")
            Return
         End If
         Try
            UdpClientObj.Send(data, data.Length, tbXmitIP.Text, Port)
         Catch ex As Exception
            MsgBox(ex.Message)
         End Try
      End If
   End Sub

   Sub CloseClient()
      UdpClientObj.Close()
      UdpClientObj = Nothing
   End Sub

#Region " Create Client "
   Private Sub cmdCreateClient_Click(sender As Object, e As EventArgs) Handles cmdCreateClient.Click
      If UdpClientObj Is Nothing Then
         ' Neues Objekt anlegen
         ' Angaben prüfen
         Dim Port As Integer = -1
         Dim IP As IPAddress = IPAddress.Any

         ' Leerezeichen entfernen
         CbLocalIP.Text = CbLocalIP.Text.Trim
         tbLocalPort.Text = tbLocalPort.Text.Trim
         tbDefaultRemoteIP.Text = tbDefaultRemoteIP.Text.Trim
         tbDefaultRemotePort.Text = tbDefaultRemotePort.Text.Trim

         ' Lokaler Endpunkt
         ' ------------------------------
         If Not cbBind.Checked Then
            UdpClientObj = New UdpClient
         Else
            If Not Int32.TryParse(tbLocalPort.Text, Port) Then
               lblError.Text = "Angabe zum lokalen Port ist ungültig."
               Return
            End If
            If Port < 0 Or Port > &HFFFF Then
               lblError.Text = "Angabe zum lokalen Port ist ungültig."
               Return
            End If

            If CbLocalIP.Text = "" Then ' nur Port angegeben
               Try
                  UdpClientObj = New UdpClient(Port)
               Catch ex As Exception
                  lblError.Text = "Fehler beim Binden an den lokalen Endpunkt:" & ex.Message
                  Return
               End Try
            Else
               If Not IPAddress.TryParse(CbLocalIP.Text, IP) Then
                  lblError.Text = "Angabe zur lokalen IP-Adresse ist ungültig."
                  Return
               End If
               Try
                  UdpClientObj = New UdpClient(New IPEndPoint(IP, Port))
               Catch ex As Exception
                  lblError.Text = "Fehler beim Binden an den lokalen Endpunkt:" & ex.Message
                  Return
               End Try
            End If
         End If

         If cbConnect.Checked Then
            If Not Int32.TryParse(tbDefaultRemotePort.Text, Port) Then
               lblError.Text = "Angabe zum Remote-Port ist ungültig."
               CloseClient()
               Return
            End If
            If Port < 1 Or Port > &HFFFF Then
               lblError.Text = "Angabe zum Remote-Port ist ungültig."
               CloseClient()
               Return
            End If
            Try
               UdpClientObj.Connect(tbDefaultRemoteIP.Text, Port)
            Catch ex As Exception
               lblError.Text = "Verbindung konnte nicht hergestellt werden."
               CloseClient()
               Return
            End Try
         End If

         grpLocalEp.Enabled = False
         grpRemoteEp.Enabled = False
         cmdCreateClient.Text = "UdpClient verwerfen"

         FillClientData()

         If cbAutoReceive.Checked Then
            If cmdReceive.Enabled Then
               CmdReceive_Click(Nothing, Nothing)
            End If
         End If
      Else
         ' Bestehendes Objekt löschen
         If Not Listener Is Nothing Then
            Listener.Stopp()
            Listener = Nothing
            Threading.Thread.Sleep(0)
         End If
         CloseClient()
         grpLocalEp.Enabled = True
         grpRemoteEp.Enabled = True
         cmdCreateClient.Text = "UdpClient erstellen"
         FillClientData()
      End If

   End Sub
#End Region

#Region " FillClientData "
   Sub FillClientData()
      If UdpClientObj Is Nothing Then
         tbClientLocalIP.Text = ""
         tbClientLocalPort.Text = ""
         cbClientLocalBound.Checked = False
         tbClientRemoteIP.Text = ""
         tbClientRemotePort.Text = ""
         cbClientRemoteConnected.Checked = False
         cmdReceive.Enabled = False
         ToolTip.SetToolTip(cmdReceive, "")
         grpXmit.Enabled = False
         cmdReceive.Text = "Empfang starten"
      Else
         grpXmit.Enabled = True
         If UdpClientObj.Client.LocalEndPoint Is Nothing Then
            tbClientLocalIP.Text = "ungebunden"
            tbClientLocalPort.Text = ""
            ToolTip.SetToolTip(cmdReceive, "Für den Empfang muss der UdpClient an den lokalen Endpunkt gebunden werden." &
               vbCrLf & "Die CheckBox 'binden' beim lokalen Endpunkt muss aktiviert sein.")
            cmdReceive.Enabled = False
         Else ' UdpClientObj ist gebunden
            With CType(UdpClientObj.Client.LocalEndPoint, IPEndPoint)
               tbClientLocalIP.Text = .Address.ToString
               tbClientLocalPort.Text = .Port.ToString
               cbClientLocalBound.Checked = True
            End With
            cmdReceive.Enabled = True
            If Listener Is Nothing Then
               ToolTip.SetToolTip(cmdReceive, "Startet den Empfangsthread.")
            Else
               ToolTip.SetToolTip(cmdReceive, "Stoppt den Empfangsthread.")
            End If
         End If
         If Not UdpClientObj.Client.Connected Then
            tbClientRemoteIP.Text = "unverbunden"
            tbClientRemotePort.Text = ""
            grpRemoteAddress.Enabled = True
         Else
            With CType(UdpClientObj.Client.RemoteEndPoint, IPEndPoint)
               tbClientRemoteIP.Text = .Address.ToString
               tbClientRemotePort.Text = .Port.ToString
               cbClientRemoteConnected.Checked = True
            End With
            grpRemoteAddress.Enabled = False
         End If
      End If
   End Sub
#End Region

   Private Sub DataChanged(sender As Object, e As EventArgs) Handles CbLocalIP.TextChanged,
      tbLocalPort.TextChanged, tbDefaultRemoteIP.TextChanged, tbDefaultRemotePort.TextChanged,
      cbBind.CheckedChanged, cbConnect.CheckedChanged
      lblError.Text = ""
   End Sub

   Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
      lblError.Text = ""
      CbLocalIP.Items.Add("0.0.0.0")
      CbLocalIP.SelectedIndex = 0
      Dim HostName As String = Dns.GetHostName()
      Dim Adresses = Dns.GetHostEntry(HostName).AddressList
      Dim LocalIPs As New List(Of String)

      For Each adr In Adresses
         If adr.AddressFamily = AddressFamily.InterNetwork Then
            CbLocalIP.Items.Add(adr.ToString)
         End If
      Next
      CbLocalIP.Items.Add("127.0.0.1")
   End Sub

   Private Sub Listener_DatagramReceived(dt As UdpResult) Handles Listener.DatagramReceived
      lbDatagrams.Items.Add(dt)
   End Sub

   Private Sub Listener_ListeningStopped() Handles Listener.ListeningStopped
      If Not Listener Is Nothing Then
         cmdReceive.Text = "Empfang starten"
         cmdReceive.Enabled = True
         Listener = Nothing
         FillClientData()
      End If
   End Sub

   Private Sub CmdAbout_Click(sender As Object, e As EventArgs) Handles CmdAbout.Click
      Dim frm As New AboutBox
      frm.ShowDialog()
   End Sub

   Private Sub CmdBroadCastAddress_Click(sender As Object, e As EventArgs) Handles CmdBroadCastAddress.Click
      tbXmitIP.Text = "255.255.255.255"
   End Sub

#Region " Tooltip für cmdReceive"
   Private CurrentToolTipControl As Control = Nothing
   Private Sub Main_MouseMove(sender As Object, e As MouseEventArgs) Handles Me.MouseMove
      Dim Control As Control = GetChildAtPoint(e.Location)
      If (Not Control Is Nothing) Then
         If Control Is cmdReceive Then
            If (Not Control.Enabled AndAlso CurrentToolTipControl Is Nothing) Then
               Dim toolTipString As String = ToolTip.GetToolTip(Control)
               ' trigger the tooltip with no delay And some basic positioning just to give you an idea
               ToolTip.Show(toolTipString, Control, CInt(Control.Width / 2), CInt(Control.Height / 2))
               CurrentToolTipControl = Control
            End If
         End If
      Else
            If (Not CurrentToolTipControl Is Nothing) Then
               ToolTip.Hide(CurrentToolTipControl)
            End If
         CurrentToolTipControl = Nothing
      End If
   End Sub

   Private Sub lbDatagrams_MouseMove(sender As Object, e As MouseEventArgs) Handles lbDatagrams.MouseMove
      Main_MouseMove(Nothing, New MouseEventArgs(Nothing, 0, 0, 0, 0))
   End Sub
#End Region
End Class