﻿Imports System.ComponentModel
Imports System.Net
Imports System.Net.Sockets

''' <summary>
''' Ein Hintergrund-Thread, der kontinuierlich auf den Eingang von UDP-Datagrammen lauscht.
''' </summary>
Public Class UdpListener
   Private WithEvents BGWorker As BackgroundWorker
   Private UdpClientObj As UdpClient
   Private SenderEP As New IPEndPoint(IPAddress.Any, 0)

   ''' <summary>Wird ausgelöst, wenn ein neues Datagramm empfnagen wurde.</summary>
   ''' <param name="Datagramm">Das empfangene Datagramm.</param>
   Public Event DatagramReceived(Datagramm As UdpResult)
   ''' <summary>Wird ausgelöst, wenn der Hintergrund-Thread beendet wurde.</summary>
   Public Event ListeningStopped()

   ''' <summary>Startet den Hintergrund-Thread.</summary>
   ''' <param name="Client">UDP-Client zum Empfang der Daten.</param>
   ''' <param name="ClearBuffer">Wenn <b>true</b> werden bereits gepufferte Daten gelöscht.</param>
   Public Sub Start(Client As UdpClient, ClearBuffer As Boolean)
      UdpClientObj = Client
      BGWorker = New BackgroundWorker
      BGWorker.WorkerReportsProgress = True
      BGWorker.WorkerSupportsCancellation = True
      If ClearBuffer Then
         While UdpClientObj.Available > 0
            Dim DataBytes = UdpClientObj.Receive(SenderEP)
         End While
      End If
         BGWorker.RunWorkerAsync()
   End Sub

   Public Sub Stopp()
      BGWorker.CancelAsync()
   End Sub

   Private Sub Listener_DoWork(sender As Object, e As DoWorkEventArgs) Handles BGWorker.DoWork
      While Not BGWorker.CancellationPending
         ' Daten empfangen und zurückmelden
         Dim u = UdpClientObj
         If UdpClientObj.Available > 0 Then
            Dim DataBytes = UdpClientObj.Receive(SenderEP) ' Liefert den Sender -> SenderEP
            Dim a = CType(UdpClientObj.Client.LocalEndPoint, IPEndPoint)
            BGWorker.ReportProgress(0, New UdpResult(DataBytes, SenderEP, a))
         End If
         Threading.Thread.Sleep(0) ' Kontrolle kurz abgeben
      End While
   End Sub

   Private Sub Listener_ProgressChanged(sender As Object, e As ProgressChangedEventArgs) Handles BGWorker.ProgressChanged
      Dim datagram = CType(e.UserState, UdpResult)
      RaiseEvent DatagramReceived(datagram)
   End Sub

   Private Sub Listener_RunWorkerCompleted(sender As Object, e As RunWorkerCompletedEventArgs) Handles BGWorker.RunWorkerCompleted
      RaiseEvent ListeningStopped()
   End Sub
End Class
